/*
WormBench - Synthetic workload for Transactinal Memory Systems
Copyright (C) 2008  Barcelona Supercomputing Center www.bscmsrc.eu

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

//+----------------------------------------------------------------------------
//
// File Name: CpuTools.cs
// Description: Cpu related tools to set the affinity of threads.
// Organization: Barcelona Supercomputing Center
// Author: Ferad Zyulkyarov ferad.zyulkyarov[@]bsc.es
// Date: 29.03.2008
//
//-----------------------------------------------------------------------------

using System;
using System.Runtime.InteropServices;
using System.Threading;

namespace System.Threading
{
    public class ThreadHelper
    {
        public static bool SetThreadAffinityToCpu(Thread thread, uint cpuId)
        {
            Console.WriteLine("WormBench SetThreadAffinityToCpu called.");
            return true;
        }
    }
}

namespace Bsc.WormBench
{
    /// <summary>
    /// Cpu related tools to set the affinity of threads.
    /// </summary>
    public class CpuTools
    {
        //
        // For now assume that we have 8 CPUS
        //
        private static uint cpusCount = 8;

        private static uint currentThreadIndex = 0;

        public static void SetCurrentThreadAffinity()
        {
            System.UIntPtr currentThreadId = GetCurrentThread();

            System.UIntPtr result = SetThreadAffinityMask(
                currentThreadId, 
                (System.UIntPtr)((currentThreadIndex % cpusCount) + 1));
            currentThreadIndex++;

            if (result == System.UIntPtr.Zero)
            {
                Console.WriteLine("Set affinifty failed: " + result);
            }
        }

        public static uint CpusCount
        {
            get
            {
                return cpusCount;
            }
            set
            {
                cpusCount = value;
            }
        }

        #region Native methods
        [DllImport("kernel32.dll")]
        
        private static extern System.UIntPtr SetThreadAffinityMask(
            System.UIntPtr threadHandle, 
            System.UIntPtr affinityMask);

        [DllImport("kernel32.dll")]
        public static extern System.UIntPtr GetCurrentThread();
        #endregion
    }
}
